@@ -67,8 +67,10 @@ def _filename_explode(x):
         if isinstance(x, pathlib.Path):
              x = str(x)
     parsed = urlparse(x)
-    if _bytesid(parsed.scheme) == b"file" or len(parsed.scheme) == 0 or (os.name == "nt" and open._windows_absolute.match(x) is not None):
-        pattern = os.path.expanduser(parsed.netloc + parsed.path)
+    if _bytesid(parsed.scheme) == b"file" or len(parsed.scheme) == 0 or (os.name == "nt" and _filename_explode._windows_absolute.match(x) is not None):
+        if not (os.name == "nt" and _filename_explode._windows_absolute.match(x) is not None):
+            path = parsed.netloc + parsed.path
+        pattern = os.path.expanduser(path)
         if "*" in pattern or "?" in pattern or "[" in pattern:
             out = sorted(glob.glob(pattern))
             if len(out) == 0:
@@ -111,8 +113,8 @@ def _normalize_entrystartstop(numentries, entrystart, entrystop):
 
 def iterate(path, treepath, branches=None, entrysteps=float("inf"), outputtype=dict, namedecode=None, reportpath=False, reportfile=False, reportentries=False, flatten=False, flatname=None, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, blocking=True, localsource=MemmapSource.defaults, xrootdsource=XRootDSource.defaults, httpsource=HTTPSource.defaults, **options):
     awkward = _normalize_awkwardlib(awkwardlib)
-    for tree, newbranches, globalentrystart, thispath, thisfile in _iterate(path, treepath, branches, awkward, localsource, xrootdsource, httpsource, **options):
-        for start, stop, arrays in tree.iterate(branches=newbranches, entrysteps=entrysteps, outputtype=outputtype, namedecode=namedecode, reportentries=True, entrystart=0, entrystop=tree.numentries, flatten=flatten, flatname=flatname, awkwardlib=awkward, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, blocking=blocking):
+    for tree, branchesinterp, globalentrystart, thispath, thisfile in _iterate(path, treepath, branches, awkward, localsource, xrootdsource, httpsource, **options):
+        for start, stop, arrays in tree.iterate(branches=branchesinterp, entrysteps=entrysteps, outputtype=outputtype, namedecode=namedecode, reportentries=True, entrystart=0, entrystop=tree.numentries, flatten=flatten, flatname=flatname, awkwardlib=awkward, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, blocking=blocking):
 
             if getattr(outputtype, "__name__", None) == "DataFrame" and getattr(outputtype, "__module__", None) == "pandas.core.frame":
                 if type(arrays.index).__name__ == "MultiIndex":
@@ -123,7 +125,13 @@ def iterate(path, treepath, branches=None, entrysteps=float("inf"), outputtype=d
                     awkward.numpy.add(index, globalentrystart, out=index)
 
                 elif type(arrays.index).__name__ == "RangeIndex":
-                    arrays.index = type(arrays.index)(arrays.index.start + globalentrystart, arrays.index.stop + globalentrystart)
+                    if hasattr(arrays.index, "start") and hasattr(arrays.index, "stop"):
+                        indexstart = arrays.index.start        # pandas>=0.25.0
+                        indexstop = arrays.index.stop
+                    else:
+                        indexstart = arrays.index._start       # pandas<0.25.0
+                        indexstop = arrays.index._stop
+                    arrays.index = type(arrays.index)(indexstart + globalentrystart, indexstop + globalentrystart)
 
                 else:
                     if hasattr(arrays.index, "array"):
@@ -150,11 +158,6 @@ def _iterate(path, treepath, branches, awkward, localsource, xrootdsource, https
     else:
         paths = [y for x in path for y in _filename_explode(x)]
 
-    oldpath = None
-    oldbranches = None
-    holdover = None
-    holdoverentries = 0
-    outerstart = 0
     globalentrystart = 0
     for path in paths:
         file = uproot.rootio.open(path, localsource=localsource, xrootdsource=xrootdsource, httpsource=httpsource, **options)
@@ -162,22 +165,11 @@ def _iterate(path, treepath, branches, awkward, localsource, xrootdsource, https
             tree = file[treepath]
         except KeyError:
             continue
-        listbranches = list(tree._normalize_branches(branches, awkward))
+        branchesinterp = OrderedDict()
+        for branch, interpretation in tree._normalize_branches(branches, awkward):
+            branchesinterp[branch.name] = interpretation
 
-        newbranches = OrderedDict((branch.name, interpretation) for branch, interpretation in listbranches)
-        if oldbranches is not None:
-            for key in set(oldbranches.keys()).union(set(newbranches.keys())):
-                if key not in newbranches:
-                    raise ValueError("branch {0} cannot be found in {1}, but it was in {2}".format(repr(key), repr(path), repr(oldpath)))
-                if key not in oldbranches:
-                    del newbranches[key]
-                elif not newbranches[key].compatible(oldbranches[key]):
-                    raise ValueError("branch {0} interpreted as {1} in {2}, but as {3} in {4}".format(repr(key), newbranches[key], repr(path), oldbranches[key], repr(oldpath)))
-
-        oldpath = path
-        oldbranches = newbranches
-
-        yield tree, newbranches, globalentrystart, path, file
+        yield tree, branchesinterp, globalentrystart, path, file
         globalentrystart += tree.numentries
 
 ################################################################ methods for TTree
@@ -232,6 +224,7 @@ class TTreeMethods(object):
         branch._isTClonesArray = isTClonesArray
         if isinstance(streamer, uproot.rootio.TStreamerSTL) and streamer._fSTLtype == uproot.const.kSTLvector:
             branch._vecstreamer = streamerinfosmap.get(re.match(self._vector_regex, streamer._fTypeName).group(1), None)
+            isTClonesArray = True
         else:
             branch._vecstreamer = None
 
@@ -495,7 +488,16 @@ class TTreeMethods(object):
                     break
 
     def array(self, branch, interpretation=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, blocking=True):
-        return self.get(branch).array(interpretation=interpretation, entrystart=entrystart, entrystop=entrystop, flatten=flatten, awkwardlib=awkwardlib, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, blocking=blocking)
+        awkward = _normalize_awkwardlib(awkwardlib)
+        branches = list(self._normalize_branches(branch, awkward))
+        if len(branches) == 1:
+            if interpretation is None:
+                tbranch, interpretation = branches[0]
+            else:
+                tbranch, _ = branches[0]
+        else:
+            raise ValueError("list of branch names or glob/regex matches more than one branch; use TTree.arrays (plural)")
+        return tbranch.array(interpretation=interpretation, entrystart=entrystart, entrystop=entrystop, flatten=flatten, awkwardlib=awkwardlib, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, blocking=blocking)
 
     def arrays(self, branches=None, outputtype=dict, namedecode=None, entrystart=None, entrystop=None, flatten=False, flatname=None, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, blocking=True):
         awkward = _normalize_awkwardlib(awkwardlib)
@@ -543,11 +545,22 @@ class TTreeMethods(object):
         else:
             return wait
 
-    def lazyarray(self, branch, interpretation=None, entrysteps=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False):
-        return self.get(branch).lazyarray(interpretation=interpretation, entrysteps=entrysteps, entrystart=entrystart, entrystop=entrystop, flatten=flatten, awkwardlib=awkwardlib, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, persistvirtual=persistvirtual)
+    def lazyarray(self, branch, interpretation=None, entrysteps=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False, chunked=True):
+        awkward = _normalize_awkwardlib(awkwardlib)
+        branches = list(self._normalize_branches(branch, awkward))
+        if len(branches) == 1:
+            if interpretation is None:
+                tbranch, interpretation = branches[0]
+            else:
+                tbranch, _ = branches[0]
+        else:
+            raise ValueError("list of branch names or glob/regex matches more than one branch; use TTree.lazyarrays (plural)")
+        return tbranch.lazyarray(interpretation=interpretation, entrysteps=entrysteps, entrystart=entrystart, entrystop=entrystop, flatten=flatten, awkwardlib=awkwardlib, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, persistvirtual=persistvirtual, chunked=chunked)
 
-    def lazyarrays(self, branches=None, namedecode="utf-8", entrysteps=None, entrystart=None, entrystop=None, flatten=False, profile=None, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False):
+    def lazyarrays(self, branches=None, namedecode="utf-8", entrysteps=None, entrystart=None, entrystop=None, flatten=False, profile=None, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False, chunked=True):
         entrystart, entrystop = _normalize_entrystartstop(self.numentries, entrystart, entrystop)
+        if not chunked and entrysteps is None:
+            entrysteps = float('inf')
         entrysteps = list(self._normalize_entrysteps(entrysteps, branches, entrystart, entrystop, keycache))
         awkward = _normalize_awkwardlib(awkwardlib)
         branches = list(self._normalize_branches(branches, awkward))
@@ -569,13 +582,17 @@ class TTreeMethods(object):
             else:
                 VirtualArray = awkward.VirtualArray
 
-            chunks = []
-            counts = []
-            for start, stop in entrysteps:
-                chunks.append(VirtualArray(lazytree, (branch.name, start, stop), cache=cache, type=awkward.type.ArrayType(stop - start, interpretation.type), persistvirtual=persistvirtual))
-                counts.append(stop - start)
             name = branch.name.decode("ascii") if namedecode is None else branch.name.decode(namedecode)
-            out[name] = awkward.ChunkedArray(chunks, counts)
+            if chunked:
+                chunks = []
+                counts = []
+                for start, stop in entrysteps:
+                    chunks.append(VirtualArray(lazytree, (branch.name, start, stop), cache=cache, type=awkward.type.ArrayType(stop - start, interpretation.type), persistvirtual=persistvirtual))
+                    counts.append(stop - start)
+                out[name] = awkward.ChunkedArray(chunks, counts)
+            else:
+                start, stop = entrysteps[0]
+                out[name] = VirtualArray(lazytree, (branch.name, start, stop), cache=cache, type=awkward.type.ArrayType(stop - start, interpretation.type), persistvirtual=persistvirtual)
 
         if profile is not None:
             out = uproot_methods.profiles.transformer(profile)(out)
@@ -1123,7 +1140,7 @@ class TBranchMethods(object):
             self._tryrecover()
         return self._threadsafe_key(i, keycache, False)._fObjlen
 
-    def basket_compressedbytes(self, i):
+    def basket_compressedbytes(self, i, keycache=None):
         if self._recoveredbaskets is None:
             self._tryrecover()
         key = self._threadsafe_key(i, keycache, False)
@@ -1177,7 +1194,7 @@ class TBranchMethods(object):
             byteoffsets[-1] = key._fLast
             awkward.numpy.subtract(byteoffsets, key._fKeylen, byteoffsets)
 
-        return interpretation.fromroot(data, byteoffsets, local_entrystart, local_entrystop)
+        return interpretation.fromroot(data, byteoffsets, local_entrystart, local_entrystop, key._fKeylen)
 
     def basket(self, i, interpretation=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None):
         awkward = _normalize_awkwardlib(awkwardlib)
@@ -1566,7 +1583,7 @@ class TBranchMethods(object):
                 raise TypeError("entrysteps must be None for cluster iteration, a positive integer for equal steps in number of entries (inf for maximal), a memory size string (number followed by B/kB/MB/GB/etc.), or an iterable of 2-tuples for explicit entry starts (inclusive) and stops (exclusive)")
             return entrysteps
 
-    def lazyarray(self, interpretation=None, entrysteps=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False):
+    def lazyarray(self, interpretation=None, entrysteps=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False, chunked=True):
         if self._recoveredbaskets is None:
             self._tryrecover()
         awkward = _normalize_awkwardlib(awkwardlib)
@@ -1574,6 +1591,8 @@ class TBranchMethods(object):
         if interpretation is None:
             raise ValueError("cannot interpret branch {0} as a Python type\n   in file: {1}".format(repr(self.name), self._context.sourcepath))
         entrystart, entrystop = _normalize_entrystartstop(self.numentries, entrystart, entrystop)
+        if not chunked and entrysteps is None:
+            entrysteps = float('inf')
         entrysteps = self._normalize_entrysteps(entrysteps, entrystart, entrystop, keycache)
 
         inner = interpretation
@@ -1591,14 +1610,18 @@ class TBranchMethods(object):
 
         lazybranch = _LazyBranch(self._context.sourcepath, self._context.treename, self.name, self, interpretation, flatten, awkward.__name__, basketcache, keycache, executor)
 
-        chunks = []
-        counts = []
-        for start, stop in entrysteps:
-            numentries = stop - start
-            chunks.append(VirtualArray(lazybranch, (start, stop), cache=cache, type=awkward.type.ArrayType(numentries, interpretation.type), persistvirtual=persistvirtual))
-            counts.append(numentries)
+        if chunked:
+            chunks = []
+            counts = []
+            for start, stop in entrysteps:
+                numentries = stop - start
+                chunks.append(VirtualArray(lazybranch, (start, stop), cache=cache, type=awkward.type.ArrayType(numentries, interpretation.type), persistvirtual=persistvirtual))
+                counts.append(numentries)
 
-        return chunkedarray(chunks, counts)
+            return chunkedarray(chunks, counts)
+        else:
+            start, stop = entrysteps[0]
+            return VirtualArray(lazybranch, (start, stop), cache=cache, type=awkward.type.ArrayType(numentries, interpretation.type), persistvirtual=persistvirtual)
 
     class _BasketKey(object):
         def __init__(self, source, cursor, compression, complete):
@@ -1925,6 +1948,8 @@ def lazyarray(path, treepath, branchname, interpretation=None, namedecode="utf-8
     else:
         branches = {branchname: interpretation}
     out = lazyarrays(path, treepath, branches=branches, namedecode=namedecode, entrysteps=entrysteps, flatten=flatten, profile=None, awkwardlib=awkwardlib, cache=cache, basketcache=basketcache, keycache=keycache, executor=executor, persistvirtual=persistvirtual, localsource=localsource, xrootdsource=xrootdsource, httpsource=httpsource, **options)
+    if len(out.columns) != 1:
+        raise ValueError("list of branch names or glob/regex matches more than one branch; use uproot.lazyarrays (plural)")
     return out[out.columns[0]]
 
 def lazyarrays(path, treepath, branches=None, namedecode="utf-8", entrysteps=float("inf"), flatten=False, profile=None, awkwardlib=None, cache=None, basketcache=None, keycache=None, executor=None, persistvirtual=False, localsource=MemmapSource.defaults, xrootdsource=XRootDSource.defaults, httpsource=HTTPSource.defaults, **options):
@@ -1998,7 +2023,7 @@ def daskframe(path, treepath, branches=None, namedecode="utf-8", entrysteps=floa
 
 ################################################################ for quickly getting numentries
 
-def numentries(path, treepath, total=True, localsource=MemmapSource.defaults, xrootdsource=XRootDSource.defaults, httpsource=HTTPSource.defaults, executor=None, blocking=True, **options):
+def numentries(path, treepath, total=True, localsource=MemmapSource.defaults, xrootdsource={"timeout": None, "chunkbytes": 32*1024, "limitbytes": 1024**2, "parallel": False}, httpsource={"chunkbytes": 32*1024, "limitbytes": 1024**2, "parallel": False}, executor=None, blocking=True, **options):
     if isinstance(path, string_types):
         paths = _filename_explode(path)
     else:
